@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: auv_texture.erl,v 1.36 2006/04/27 13:46:55 dgud Exp $
+%%     $Id: auv_texture.erl,v 1.37 2006/08/14 08:45:19 dgud Exp $
 %%
 
 -module(auv_texture).
@@ -1043,19 +1043,19 @@ pass({auv_faces, [Type]},_) ->
 pass({auv_faces, _},Sh) ->
     pass({auv_faces,?OPT_FACES},Sh);
 pass({auv_light, _},_Sh) ->
-    Circle = fun(N, Y, R) ->
-		     Delta = math:pi()*2 / N,
-		     [{R*math:cos(I*Delta), Y, R*math:sin(I*Delta)} 
-		      || I <- lists:seq(0, N-1)]
-	     end,
-    Sphere = fun(Ns,Nl) ->
-		     Delta = math:pi() / Nl,
-		     PosAndRads= [{math:cos(I*Delta), math:sin(I*Delta)} 
-				  || I <- lists:seq(1, Nl-1)],
-		     Circles = [Circle(Ns, Pos, Rad) || {Pos, Rad} <- PosAndRads],
-		     lists:flatten(Circles)
-	     end,
-    %Lights = Sphere(8,4) ++ [{0.0,1.0,0.0}, {0.0,-1.0,0.0}],
+%%     Circle = fun(N, Y, R) ->
+%% 		     Delta = math:pi()*2 / N,
+%% 		     [{R*math:cos(I*Delta), Y, R*math:sin(I*Delta)} 
+%% 		      || I <- lists:seq(0, N-1)]
+%% 	     end,
+%%     Sphere = fun(Ns,Nl) ->
+%% 		     Delta = math:pi() / Nl,
+%% 		     PosAndRads= [{math:cos(I*Delta), math:sin(I*Delta)} 
+%% 				  || I <- lists:seq(1, Nl-1)],
+%% 		     Circles = [Circle(Ns, Pos, Rad) || {Pos, Rad} <- PosAndRads],
+%% 		     lists:flatten(Circles)
+%% 	     end,
+%%  Lights = Sphere(8,4) ++ [{0.0,1.0,0.0}, {0.0,-1.0,0.0}],
     Ligths = [{1.0,0.0,0.0}],
     
     VsS = wings_gl:compile(vertex, read_file("light1.vs")),
@@ -1147,7 +1147,7 @@ shader_pass({value,#sh{args=Args,tex_units=TexUnits,reqs=Reqs}},
 
 shader_uniforms([{uniform,color,Name,_,_}|As],[Val|Opts],Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
-    gl:uniform4fv(Loc,1,Val),
+    gl:uniform4fv(Loc,1,[Val]),
     shader_uniforms(As,Opts,Conf);
 shader_uniforms([{uniform,float,Name,_,_}|As],[Val|Opts],Conf) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
@@ -1156,7 +1156,6 @@ shader_uniforms([{uniform,float,Name,_,_}|As],[Val|Opts],Conf) ->
 shader_uniforms([{uniform,menu,Name,_,_}|As],[Vals|Opts],Conf) 
   when is_list(Vals) ->
     Loc = wings_gl:uloc(Conf#sh_conf.prog,Name),
-    %% Bug in esdl gl uniformXfv can't send arrays of values.
     foldl(fun(Val,Cnt) -> gl:uniform1f(Loc+Cnt,Val),Cnt+1 end,0,Vals),
     shader_uniforms(As,Opts,Conf);
 
@@ -1259,7 +1258,6 @@ render_lights([{Lx,Ly,Lz}| Ligths], PM, Dist, C={Cx,Cy,Cz}, LP1,Ts = #ts{charts=
     gl:matrixMode(?GL_PROJECTION),  gl:pushMatrix(),  gl:loadMatrixd(PM),
     gl:matrixMode(?GL_MODELVIEW),   gl:pushMatrix(),  gl:loadIdentity(),
     glu:lookAt(Lx*Dist,Ly*Dist,Lz*Dist,Cx,Cy,Cz,0.0,1.0,0.0),
-    MV = gl:getDoublev(?GL_MODELVIEW_MATRIX),
     gl:clear(?GL_DEPTH_BUFFER_BIT bor ?GL_COLOR_BUFFER_BIT),
 %%    gl:enable(?GL_CULL_FACE),
     gl:enable(?GL_DEPTH_TEST), 
