@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wpc_autouv.erl,v 1.317 2005/10/19 18:00:15 dgud Exp $
+%%     $Id: wpc_autouv.erl,v 1.327 2006/03/14 13:52:20 dgud Exp $
 %%
 
 -module(wpc_autouv).
@@ -27,15 +27,12 @@
 		member/2,foreach/2,keysearch/3]).
 
 %% Exports to auv_texture.
--export([has_texture/2]).
+%%-export([has_texture/2]).
 
 %% Exports to auv_seg_ui.
 -export([init_show_maps/4]).
 
-%% Background Image compressed.
-
-init() ->
-    true.
+init() -> true.
 
 menu({body}, Menu) ->
     case wpc_snap:active() of
@@ -62,19 +59,21 @@ auv_menu() ->
     {?__(1,"UV Mapping"), {?MODULE, fun auv_menu/2}}.
 auv_menu(help,_) ->
     {?__(2,"Generate UV mapping or texture"),
-     "",
-     ?__(3,"Force to segmenting mode")};
+     ?__(25,"Re-segment object(s)"),
+     ?__(3,"Force to segmenting mode (delete old segments)")};
 auv_menu(1,_What) -> 
     case wings_pref:get_value(advanced_menus) of
 	false -> 
 	    [{?__(4,"Direct"), segment,
 	      ?__(5,"Open UV-window directly if selection already contains uv-coords")},
-	     {?__(6,"Force Segment"), force_seg,
+	     {?__(8,"Force Segment (keep previous)"), segment_old,
+	      ?__(9,"Re-segment with old UV-coords ")},
+	     {?__(6,"Force Segment (delete previous)"), force_seg,
 	      ?__(7,"Delete old UV-coords and start over with segmenting")}];
 	true -> 
 	    {?MODULE, segment}
     end;
-auv_menu(2,_) -> ignore;
+auv_menu(2,_) -> {?MODULE, segment_old};
 auv_menu(3,_) -> {?MODULE, force_seg}.
 
 command({body,{?MODULE, Op}} , St) ->
@@ -84,7 +83,7 @@ command({face,{?MODULE, Op}} , St) ->
 command({?MODULE, Op}, St) ->
     start_uvmap(Op, St);
 command({window,uv_editor_window}, St) ->
-        start_uvmap(edit, St);
+    start_uvmap(edit, St);
 command(_Cmd, _) -> 
     next.
 
@@ -97,18 +96,17 @@ start_uvmap_1([{Id,_}|T], Action, St) ->
     EditExists = wings_wm:is_window(EditWin),
     SegWin    = {autouv,{segment,Id}},
     SegExists = wings_wm:is_window(SegWin),
-    
     case segment_or_edit(Action,Id,St) of
 	{edit,Fs} when EditExists ->
 	    wings_wm:send(EditWin, {add_faces,Fs,St}),
 	    wings_wm:raise(EditWin);
-	{seg_ui,Fs} when SegExists ->
+	{seg_ui,Fs,_} when SegExists ->
 	    wings_wm:send(SegWin, {add_faces,Fs,St}),
 	    wings_wm:raise(SegWin);
 	Op when element(1,Op) == edit ->	    
-	    create_window(Op, EditWin, Id, St);
+	    create_window(Op, EditWin, Id, St,St);
 	Op ->	    
-	    create_window(Op, SegWin, Id, St)
+	    create_window(Op, SegWin, Id, St,St)
     end,
     start_uvmap_1(T, Action, St);
 start_uvmap_1([], _, _) -> keep.
@@ -119,39 +117,45 @@ segment_or_edit(segment,Id,#st{selmode=face,sel=Sel,shapes=Shs}) ->
     UVFs = gb_sets:from_ordset(wings_we:uv_mapped_faces(We)),
     {value,{_,Fs}} = lists:keysearch(Id, 1, Sel),
     case gb_sets:is_subset(Fs,UVFs) of
-	false -> {seg_ui, Fs};
+	false -> {seg_ui,Fs,delete_old};
 	true ->  {edit, Fs}
     end;
 segment_or_edit(segment,Id,#st{shapes=Shs}) ->
     We = gb_trees:get(Id, Shs),
     case wings_we:uv_mapped_faces(We) of    
-	[] -> {seg_ui,object};
+	[] -> {seg_ui,object,delete_old};
 	_ ->  {edit,object}
     end;
 segment_or_edit(force_seg,Id,#st{selmode=face,sel=Sel}) -> 
     {value,{_,Fs}} = lists:keysearch(Id, 1, Sel),
-    {seg_ui,Fs};
+    {seg_ui,Fs,delete_old};
 segment_or_edit(force_seg,_Id,_) ->
-    {seg_ui,object}.
+    {seg_ui,object,delete_old};
+segment_or_edit(segment_old,Id,#st{selmode=face,sel=Sel}) -> 
+    {value,{_,Fs}} = lists:keysearch(Id, 1, Sel),
+    {seg_ui,Fs,keep_old};
+segment_or_edit(segment_old,_Id,_) ->
+    {seg_ui,object,keep_old}.
 	    
-create_window(Action, Name, Id, #st{shapes=Shs}=St) ->
+create_window(Action, Name, Id, #st{shapes=Shs}=St, OrigSt) ->
     #we{name=ObjName} = We = gb_trees:get(Id, Shs),
     Op = {replace,fun(Ev) -> auv_event(Ev, St) end},
     Segment = if element(1,Action) == edit -> ""; true -> ?__(1,"Segmenting") end,
     Title = "AutoUV "++ Segment ++": " ++ ObjName,
-    {X,Y,W,H} = init_drawarea(),
+    {X0,Y,W,H} = init_drawarea(),
     Props = [{display_lists,Name}|wings_view:initial_properties()],
     CreateToolbar = fun(N, P, Wi) -> wings_toolbar:create(N, P, Wi) end,
+    X = if element(1,Action) == edit -> X0; true -> 10 end,
     wings_wm:toplevel(Name, Title, {X,Y,highest}, {W,H},
 		      [resizable,closable,menubar,{properties,Props},
 		       {toolbar,CreateToolbar}], Op),
-    wings_wm:send(Name, {init,{Action,We}}).
+    wings_wm:send(Name, {init,{Action,We},OrigSt}).
 
-auv_event({init,Op}, St) ->
+auv_event({init,Op,OrigSt}, St) ->
     wings:init_opengl(St),
     case Op of
-	{{edit,What},We} -> start_edit(What, We, St);
-	{{seg_ui,_},We} ->  auv_seg_ui:start(We, We, St)
+	{{edit,What},We} -> start_edit(What, We, St, OrigSt);
+	{{seg_ui,_,Oper},We} ->  auv_seg_ui:start(Oper, We, We, St)
     end;
 auv_event(redraw, _) ->
     wings_wm:clear_background(),
@@ -165,38 +169,35 @@ auv_event(_Ev, _) -> keep.
 %%% Start the UV editor.
 %%%
 
-start_edit(Mode, We, St) ->
+start_edit(Mode, We, St, OrigSt) ->
     MatNames0 = wings_facemat:all(We),
     MatNames1 = sofs:from_external(MatNames0, [{face,material}]),
     MatNames2 = sofs:converse(MatNames1),
     MatNames3 = sofs:relation_to_family(MatNames2),
     MatNames4 = sofs:to_external(MatNames3),
-    MatNames = [Mat || {Name,_}=Mat <- MatNames4, has_texture(Name, St)],
+    MatNames = [Mat || {Name,_}=Mat <- MatNames4, get_texture(Name, St) /= false],
     case MatNames of
 	[{MatName,_}] ->
-	    do_edit(MatName, Mode, We, St);
+	    do_edit(MatName, Mode, We, St, OrigSt);
 	_ ->
-	    do_edit(none, Mode, We, St)
+	    do_edit(none, Mode, We, St,OrigSt)
     end.
 
-do_edit(MatName, Mode, #we{id=Id}=We, #st{shapes=Shs0}=GeomSt) ->
-    Shs = gb_trees:update(Id, We#we{fs=undefined,es=gb_trees:empty()}, Shs0),
-    FakeGeomSt = GeomSt#st{sel=[],shapes=Shs},
-    AuvSt = create_uv_state(gb_trees:empty(), MatName, Mode, We, FakeGeomSt),
+do_edit(MatName, Mode, We, GeomSt, OrigSt) ->
+    AuvSt = create_uv_state(gb_trees:empty(), MatName, Mode, We, GeomSt,OrigSt),
     new_geom_state(GeomSt, AuvSt).
 
-init_show_maps(Charts0, Fs, OldWe=#we{name=WeName,id=Id}, GeomSt0) ->
+init_show_maps(Charts0, Fs, #we{name=WeName,id=Id}, GeomSt0) ->
     Charts1 = auv_placement:place_areas(Charts0),
     Charts  = gb_trees:from_orddict(keysort(1, Charts1)),
-    ExistingMats = wings_facemat:used_materials(OldWe),
     MatName0 = list_to_atom(WeName++"_auv"),
     {GeomSt1,MatName} = 
-	case lists:member(MatName0,ExistingMats) of
+	case gb_trees:is_defined(MatName0,GeomSt0#st.mat) of
 	    true -> 
 		{GeomSt0,MatName0};
 	    false ->
-		Tx = bg_image(),
-		add_material(Tx, WeName, none, GeomSt0)
+		Tx = bg_img_id(),
+		add_material(Tx, WeName, GeomSt0)
 	end,
     GeomSt = insert_initial_uvcoords(Charts, Id, MatName, GeomSt1),
     EditWin = {autouv,Id},
@@ -205,20 +206,30 @@ init_show_maps(Charts0, Fs, OldWe=#we{name=WeName,id=Id}, GeomSt0) ->
 	    wings_wm:send(EditWin, {add_faces,Fs,GeomSt}),
 	    wings_wm:send(geom, {new_state,GeomSt});
 	false ->
-	    create_window({edit,Fs},EditWin,Id,GeomSt),
+	    create_window({edit,Fs},EditWin,Id,GeomSt,GeomSt0),
 	    wings_wm:send(geom, {new_state,GeomSt})
     end,
-    cleanup_before_exit(),
-    delete.
+    GeomSt.
 
-create_uv_state(Charts, MatName, Fs, We, GeomSt) ->
+create_uv_state(Charts, MatName, Fs, We, GeomSt = #st{shapes=Shs0},OrigSt) ->
     wings:mode_restriction([vertex,edge,face,body]),
     wings_wm:current_state(#st{selmode=body,sel=[]}),
-    Uvs = #uvstate{st=wpa:sel_set(face, [], GeomSt),
-		   id=We#we.id,
-		   mode=Fs,
-		   matname=MatName},
-    St = GeomSt#st{selmode=body,sel=[],shapes=Charts,bb=Uvs},
+
+    Shs = gb_trees:update(We#we.id, We#we{fs=undefined,es=gb_trees:empty()}, Shs0),
+    FakeGeomSt = GeomSt#st{sel=[],shapes=Shs},
+
+    Image = case get_texture(MatName,GeomSt) of
+		false -> bg_img_id();
+		ImId -> ImId
+	    end,
+    Uvs = #uvstate{st=wpa:sel_set(face, [], FakeGeomSt),
+		   id      = We#we.id,
+		   mode    = Fs,
+		   bg_img  = Image,
+		   orig_st = OrigSt,
+		   matname = MatName},
+    St = FakeGeomSt#st{selmode=body,sel=[],shapes=Charts,bb=Uvs,
+		       repeatable=ignore,ask_args=none,drag_args=none},
     Name = wings_wm:this(),
 
     View = #view{origin={0.0,0.0,0.0},
@@ -255,7 +266,7 @@ create_uv_state(Charts, MatName, Fs, We, GeomSt) ->
 				 fun draw_background/1),
     wings_wm:menubar(Win, menubar()),
     wings_wm:send({menubar,Win}, {current_state,St}),
-
+    
     St.
 
 menubar() ->
@@ -263,7 +274,13 @@ menubar() ->
       fun(_) ->
 	      [{?__(2,"Undo/Redo"),undo_toggle,?__(3,"Undo or redo the last command")},
 	       {?__(4,"Redo"),redo,?__(5,"Redo the last command that was undone")},
-	       {?__(6,"Undo"),undo,?__(7,"Undo the last command")}]
+	       {?__(6,"Undo"),undo,?__(7,"Undo the last command")},
+	       {?__(71,"Repeat"),repeat,?__(72,"Repeat the last command")},
+	       {?__(74,"Repeat Args"),repeat_args,
+		?__(75,"Repeat the last command with same arguments")},
+	       {?__(76,"Repeat Drag"),repeat_drag,
+		?__(77,"Repeat the last command with same arguments the same distance")}
+	      ]
       end},
      {?__(8,"View"),view,
       fun(_) ->
@@ -373,17 +390,17 @@ insert_material(Cs, MatName, We) ->
 
 %%%%% Material handling
 
-has_texture(MatName, #st{mat=Materials}) ->
-    has_texture(MatName, Materials);
-has_texture(MatName, Materials) ->
+get_texture(MatName, #st{mat=Materials}) ->
+    get_texture(MatName, Materials);
+get_texture(MatName, Materials) ->
     case gb_trees:lookup(MatName, Materials) of
 	none -> false;
 	{value,Mat} ->
 	    Maps = proplists:get_value(maps, Mat, []),
-	    none /= proplists:get_value(diffuse, Maps, none)
+	    proplists:get_value(diffuse, Maps, false)
     end.
 
-add_material(#e3d_image{}=Tx, Name, none, St0) ->
+add_material(Tx, Name, St0) ->
     MatName0 = list_to_atom(Name++"_auv"),
     Mat = {MatName0,[{opengl,[]},{maps,[{diffuse,Tx}]}]},
     case wings_material:add_materials([Mat], St0) of
@@ -391,11 +408,30 @@ add_material(#e3d_image{}=Tx, Name, none, St0) ->
 	    {St,MatName0};
 	{St,[{MatName0,MatName}]} ->
 	    {St,MatName}
-    end;
-add_material(Im = #e3d_image{}, _, MatName,St) ->
+    end.
+update_texture(Im = #e3d_image{},MatName,St) ->
     catch wings_material:update_image(MatName, diffuse, Im, St),
     {St,MatName}.
-   
+
+bg_img_id() ->
+    Is = wings_image:images(),
+    case [ImId || {ImId,#e3d_image{name="auvBG"}} <- Is] of
+	[ImId] -> ImId;
+	_ -> wings_image:new("auvBG",bg_image())
+    end.
+  
+change_texture_id(NewId, MatName, GeomSt0=#st{mat=Materials}) -> 
+    case gb_trees:lookup(MatName, Materials) of
+	none -> 
+	    GeomSt0;
+	{value,Mat0} ->
+	    Maps0 = proplists:get_value(maps, Mat0, []),
+	    Maps = lists:keyreplace(diffuse,1,Maps0,{diffuse,NewId}),
+	    Mat  = lists:keyreplace(maps,1,Mat0,{maps,Maps}),
+	    GeomSt0#st{mat=gb_trees:update(MatName,Mat,Materials)}
+    end.
+    
+
 %%%% Menus.
 
 command_menu(body, X, Y) ->
@@ -450,7 +486,10 @@ command_menu(face, X, Y) ->
 	    {basic,separator},
 	    {?__(47,"Move"),move,?__(48,"Move selected faces"),[magnet]},
 	    {?__(49,"Scale"),{scale,Scale},?__(50,"Scale selected faces"), [magnet]},
-	    {?__(51,"Rotate"),rotate,?__(52,"Rotate selected faces"), [magnet]}
+	    {?__(51,"Rotate"),rotate,?__(52,"Rotate selected faces"), [magnet]},
+	    separator,
+	    {?__(521,"Project-Unfold"),	proj_lsqcm,
+	     ?__(522,"Project selected faces from normal and unfold the rest of chart")}
 	   ] ++ option_menu(),
     wings_menu:popup_menu(X,Y, auv, Menu);
 command_menu(edge, X, Y) ->
@@ -640,32 +679,32 @@ handle_event_3({drop,_,DropData}, St) ->
     handle_drop(DropData, St);
 handle_event_3({action,{auv,create_texture}},_St) ->
     auv_texture:draw_options();
+handle_event_3({action,{auv,{draw_options,restart}}}, _St) ->
+    auv_texture:draw_options();
 handle_event_3({action,{auv,{draw_options,Opt}}}, #st{bb=Uvs}=St) ->
-    #uvstate{st=GeomSt0,matname=MatName0} = Uvs,
+    #uvstate{st=GeomSt0,matname=MatName0,bg_img=Image} = Uvs,
     Tx = ?SLOW(auv_texture:get_texture(St, Opt)),
     case MatName0 of 
 	none -> 
-	    Id = wings_image:new("UVmap", Tx),
-	    wings_image:window(Id),
+	    wings_image:update(Image, Tx),
+	    put({?MODULE,show_background}, true),
 	    get_event(St);
 	_ ->
+	    TexName = case get_texture(MatName0, St) of
+			  false -> atom_to_list(MatName0);
+			  Old  -> 
+			      OldE3d = wings_image:info(Old), 
+			      case OldE3d#e3d_image.name of
+				  "auvBG" -> atom_to_list(MatName0);
+				  Other -> Other
+			      end
+		      end,
 	    {GeomSt,MatName} = 
-		add_material(Tx#e3d_image{name=atom_to_list(MatName0)}, 
-			     undefined, MatName0, GeomSt0),
+		update_texture(Tx#e3d_image{name=TexName}, 
+			       MatName0, GeomSt0),
 	    wings_wm:send(geom, {new_state,GeomSt}),
 	    get_event(St#st{bb=Uvs#uvstate{st=GeomSt,matname=MatName}})
     end;
-handle_event_3({action,{auv,{remap,Method}}}, St0) ->
-    St = remap(Method, St0),
-    get_event(St);
-handle_event_3({action,{auv,lsqcm}}, St0) ->
-    St = reunfold(lsqcm,St0),
-    get_event(St);
-handle_event_3({action,{auv,sphere}}, St0) ->
-    St = reunfold(sphere,St0),
-    get_event(St);
-
-
 %% Others
 handle_event_3({vec_command,Command,_St}, _) when is_function(Command) ->
     %% Use to execute command with vector arguments (see wings_vec.erl).
@@ -696,6 +735,12 @@ handle_event_3({action,{edit,undo}}=Act, _) ->
     wings_wm:send(geom, Act);
 handle_event_3({action,{edit,redo}}=Act, _) ->
     wings_wm:send(geom, Act);
+handle_event_3({action,{edit,repeat}}, St) ->
+    repeat(command, St);
+handle_event_3({action,{edit,repeat_args}}, St) ->
+    repeat(args, St);
+handle_event_3({action,{edit,repeat_drag}}, St) ->
+    repeat(drag, St);
 handle_event_3({action,{view,toggle_background}}, _) ->
     Old = get({?MODULE,show_background}),
     put({?MODULE,show_background},not Old),
@@ -720,6 +765,12 @@ handle_event_3({action,Ev}, St) ->
 	{view,Cmd} when Cmd == frame ->
 	    wings_view:command(Cmd,St),
 	    get_event(St);
+	{edit, repeat} ->
+	    repeat(command, St);
+	{edit, repeat_args} ->
+	    repeat(args, St);
+	{edit,repeat_drag} ->
+	    repeat(drag, St);
 	_ ->
 %%	    io:format("Miss Action ~p~n", [Ev]),
 	    keep
@@ -770,16 +821,43 @@ new_state(#st{bb=#uvstate{}=Uvs}=St0) ->
     St = update_selected_uvcoords(St1),
     get_event(St).
 
-handle_command(move, St) ->
-    drag(wings_move:setup(free_2d, St));
-handle_command({move,Magnet}, St) ->
-    drag(wings_move:setup({free_2d,Magnet}, St));
-handle_command({scale,Dir}, St0) %% Maximize chart
+handle_command(Cmd, St0) ->    
+    case handle_command_1(Cmd,remember_command(Cmd,St0)) of
+	Drag = {drag, _} -> do_drag(Drag);
+	Result -> Result
+    end.
+
+handle_ask(Cmd, St) ->
+%%    io:format("Handle Ask ~p ~n",[Cmd]),
+    do_drag(handle_command_1(Cmd,St)).
+
+handle_command_1({'ASK',Ask}, St) ->
+    wings:ask(Ask, St, fun handle_ask/2);
+handle_command_1({remap,Method}, St0) ->
+    St = remap(Method, St0),
+    get_event(St);
+handle_command_1(M=proj_lsqcm, St0) ->
+    St = remap(M,St0),
+    get_event(St);
+handle_command_1(lsqcm, St0) ->
+    St = reunfold(lsqcm,St0),
+    get_event(St);
+handle_command_1(sphere, St0) ->
+    St = reunfold(sphere,St0),
+    get_event(St);
+handle_command_1(move, St) ->
+    wings_move:setup(free_2d, St);
+handle_command_1({move,{move,Magnet}}, St) ->
+    wings_move:setup({free_2d,Magnet}, St); %% Repeat drag with mag
+handle_command_1({move,Magnet}, St) ->
+%%    do_drag(wings_move:setup({free_2d,Magnet}, St),none);
+    wings_move:setup({free_2d,Magnet}, St);
+handle_command_1({scale,Dir}, St0) %% Maximize chart
   when Dir == max_uniform; Dir == max_x; Dir == max_y -> 
     St1 = wpa:sel_map(fun(_, We) -> stretch(Dir,We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({scale,normalize}, St0) -> %% Normalize chart sizes    
+handle_command_1({scale,normalize}, St0) -> %% Normalize chart sizes    
     #st{shapes=Sh0, bb=#uvstate{id=Id,st=#st{shapes=Orig}}} = St0,
     OWe = gb_trees:get(Id, Orig), 
     {TA2D,TA3D,List} = wings_sel:fold(fun(_,We,Areas) -> 
@@ -799,64 +877,64 @@ handle_command({scale,normalize}, St0) -> %% Normalize chart sizes
     Sh = lists:foldl(Scale, Sh0, List),
     St = update_selected_uvcoords(St0#st{shapes=Sh}),
     get_event(St);
-handle_command({scale, {'ASK', Ask}}, St) ->
+handle_command_1({scale, {'ASK', Ask}}, St) ->
     wings:ask(Ask, St, fun({Dir,M},St0) -> 
-			       drag(wings_scale:setup({Dir,center,M}, St0)) 
+			       do_drag(wings_scale:setup({Dir,center,M}, St0))
 		       end);
-handle_command({scale,Dir}, St) ->
-    drag(wings_scale:setup({Dir,center}, St));
-handle_command(rotate, St) ->
-    drag(wings_rotate:setup({free,center}, St));
-handle_command({rotate,free}, St) ->
-    drag(wings_rotate:setup({free,center}, St));
-handle_command({rotate, {'ASK', Ask}}, St) ->
+handle_command_1({scale,{Dir,M}}, St) when element(1,M) == magnet ->
+    wings_scale:setup({Dir,center,M}, St); % For repeat drag
+handle_command_1({scale,Dir}, St) ->
+    wings_scale:setup({Dir,center}, St);
+handle_command_1(rotate, St) ->
+    wings_rotate:setup({free,center}, St);
+handle_command_1({rotate,free}, St) ->
+    wings_rotate:setup({free,center}, St);
+handle_command_1({rotate, {'ASK', Ask}}, St) ->
     wings:ask(Ask, St, fun({Dir,M},St0) -> 
-			       drag(wings_rotate:setup({Dir,center,M}, St0)) 
+			       do_drag(wings_rotate:setup({Dir,center,M}, St0)) 
 		       end);
-handle_command({rotate, Magnet}, St) when element(1, Magnet) == magnet ->
-    drag(wings_rotate:setup({free,center,Magnet}, St));
-handle_command({rotate,Dir}, St0) 
+handle_command_1({rotate, Magnet}, St) when element(1, Magnet) == magnet ->
+    wings_rotate:setup({free,center,Magnet}, St); % For repeat drag
+handle_command_1({rotate,Dir}, St0) 
   when Dir == align_y; Dir == align_x; Dir == align_xy ->
     St1 = align_chart(Dir, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({rotate,Deg}, St0) ->
+handle_command_1({rotate,Deg}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> rotate_chart(Deg, We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({move_to,Dir}, St0) ->
+handle_command_1({move_to,Dir}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> move_to(Dir,We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({flip,horizontal}, St0) ->
+handle_command_1({flip,horizontal}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_horizontal(We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command({flip,vertical}, St0) ->
+handle_command_1({flip,vertical}, St0) ->
     St1 = wpa:sel_map(fun(_, We) -> flip_vertical(We) end, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command(slide, St) ->
-    drag(wings_edge_cmd:command(slide,St));
-handle_command({equal,Op}, St0) ->
+handle_command_1(slide, St) ->
+    wings_edge_cmd:command(slide,St);
+handle_command_1({equal,Op}, St0) ->
     St1 = equal_length(Op,St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command(tighten, St) ->
+handle_command_1(tighten, St) ->
     tighten(St);
-handle_command({tighten,Magnet}, St) ->
+handle_command_1({tighten,Magnet}, St) ->
     tighten(Magnet, St);
-handle_command(delete, St) ->
+handle_command_1(delete, St) ->
     get_event(delete_charts(St));
-handle_command(hide, St) ->
+handle_command_1(hide, St) ->
     get_event(hide_charts(St));
-handle_command({'ASK',Ask}, St) ->
-    wings:ask(Ask, St, fun handle_command/2);
-handle_command({flatten, Plane}, St0 = #st{selmode=vertex}) ->
+handle_command_1({flatten, Plane}, St0 = #st{selmode=vertex}) ->
     {save_state, St1} = wings_vertex_cmd:flatten(Plane, St0),
     St = update_selected_uvcoords(St1),
     get_event(St);
-handle_command(stitch, St0 = #st{selmode=edge}) ->
+handle_command_1(stitch, St0 = #st{selmode=edge}) ->
     Es = wpa:sel_fold(fun(Es,We=#we{name=#ch{emap=Emap},es=Etab},A) ->
 			      Vis = gb_sets:from_list(wings_we:visible(We)),
 			      [auv_segment:map_edge(E,Emap)
@@ -872,7 +950,7 @@ handle_command(stitch, St0 = #st{selmode=edge}) ->
     %% Do something here, i.e. restart uvmapper.
     St = rebuild_charts(gb_trees:get(Id,Geom#st.shapes), AuvSt, []),
     get_event(St);
-handle_command(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
+handle_command_1(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
     Es = wpa:sel_fold(fun(Es,We=#we{name=#ch{emap=Emap},es=Etab},A) ->
 			      Vis = gb_sets:from_list(wings_we:visible(We)),
 			      A ++ [auv_segment:map_edge(E,Emap)
@@ -890,10 +968,74 @@ handle_command(cut_edges, St0 = #st{selmode=edge,bb=#uvstate{id=Id,st=Geom}}) ->
     St  = update_selected_uvcoords(St2),
     get_event(St);
 
-handle_command(_Cmd, #st{sel=[]}) ->
-%%    io:format("~p ~n", [_Cmd]),
+handle_command_1(_Cmd, #st{sel=[]}) ->
+    io:format("Error unknown command ~p ~n", [_Cmd]),
     keep.
 
+remember_command(Cmd,St0) ->
+    St0#st{repeatable=Cmd,ask_args=none,drag_args=none}.
+
+repeat(_Type, #st{sel=Sel,repeatable=Rep}) when Sel == []; Rep == ignore ->
+    keep;
+repeat(Type, St=#st{selmode=Mode, repeatable=Cmd}) ->
+%%     io:format("Repeat ~p ~n", [{St#st.repeatable,St#st.ask_args,St#st.drag_args}]),
+    case repeatable(Cmd,Mode) of
+	false -> 
+%% 	    io:format("Not repeatable ~p ~p ~p~n",[Type,Cmd,Mode]),
+	    keep;
+	true ->
+%% 	    io:format("Repeatable ~p ~p ~p ~n",[Type,Cmd,Mode]),
+	    repeat(Type, Cmd, St)
+    end.
+
+repeat(command, Cmd, St) ->
+    repeat2(Cmd,St,none);
+repeat(args, Cmd0, St = #st{ask_args=AskArgs}) ->
+    Cmd =replace_ask(Cmd0, AskArgs),
+    repeat2(Cmd,St,none);
+repeat(drag, Cmd0, St = #st{ask_args=AskArgs,drag_args=DragArgs}) ->
+    Cmd = replace_ask(Cmd0, AskArgs),
+    repeat2(Cmd,St,DragArgs).
+
+repeat2(Cmd,St,DragArgs) ->
+    case handle_command_1(Cmd,St) of
+	{drag,Drag} -> wings_drag:do_drag(Drag, DragArgs); 
+	Other -> Other
+    end.
+
+replace_ask(Term, none) -> Term;
+replace_ask({'ASK',_}, AskArgs) -> AskArgs;
+replace_ask(Tuple0, AskArgs) when is_tuple(Tuple0) ->
+    Tuple = [replace_ask(El, AskArgs) || El <- tuple_to_list(Tuple0)],
+    list_to_tuple(Tuple);
+replace_ask(Term, _) -> Term.
+
+repeatable({remap,_},Mode) -> Mode == body;
+repeatable(lsqcm, Mode) -> Mode == vertex;
+repeatable(sphere, Mode)-> Mode == vertex;
+repeatable(proj_lsqcm, Mode) -> Mode == face;
+repeatable({scale, Dir}, Mode) 
+  when ((Dir == max_uniform) or (Dir == max_x) or (Dir == max_y) or 
+	(Dir == normalize)) and (Mode /= body) -> false;
+repeatable({rotate, Dir}, Mode) 
+  when ((Dir == align_y) or (Dir == align_x) or (Dir == align_xy)) 
+       and ((Mode == body) or (Mode == face)) -> no;
+repeatable({move_to,_},Mode) -> Mode == body;
+repeatable({flip,_},Mode) -> Mode == body;
+repeatable(slide, Mode) -> Mode == edge;
+repeatable({equal,_}, Mode) -> Mode == edge;
+repeatable(tighten, Mode) ->
+    (Mode == vertex) orelse (Mode == body);
+repeatable({tighten,_}, Mode) -> 
+    (Mode == vertex) orelse (Mode == body);
+repeatable(delete, Mode) -> Mode == body;
+repeatable(hide, Mode) -> Mode == body;
+repeatable(flatten, Mode) -> Mode == edge;
+repeatable(stitch, Mode) ->  Mode == edge;
+repeatable(cut_edges, Mode) -> Mode == edge;
+repeatable(_Cmd,_Mode) ->
+    true.
+
 fake_selection(St) ->
     wings_dl:fold(fun(#dlo{src_sel=none}, S) ->
 			  %% No selection, try highlighting.
@@ -934,10 +1076,12 @@ add_faces(NewFs,St0=#st{bb=ASt=#uvstate{id=Id,mode=Mode,st=GeomSt=#st{shapes=Shs
 	    end
     end.
 
-drag({drag,Drag}) ->
+do_drag({drag,Drag}) -> 
     wings:mode_restriction([vertex,edge,face,body]),
     wings_wm:set_prop(show_info_text, true),
-    wings_drag:do_drag(Drag, none).
+    wings_drag:do_drag(Drag,none);
+do_drag(Other) ->
+    Other.
 
 tighten(#st{selmode=vertex}=St) ->
     tighten_1(fun vertex_tighten/3, St);
@@ -946,8 +1090,7 @@ tighten(#st{selmode=body}=St) ->
 
 tighten_1(Tighten, St) ->    
     Tvs = wings_sel:fold(Tighten, [], St),
-    {drag,Drag} = wings_drag:setup(Tvs, [percent], St),
-    wings_drag:do_drag(Drag, none).
+    wings_drag:setup(Tvs, [percent], St).
 
 vertex_tighten(Vs0, We, A) ->
     Vis = gb_sets:from_ordset(wings_we:visible(We)),
@@ -964,8 +1107,7 @@ tighten(Magnet, St) ->
 				 mag_vertex_tighten(Vs, We, Magnet, A)
 			 end, [], St),
     Flags = wings_magnet:flags(Magnet, []),
-    {drag,Drag} = wings_drag:setup(Tvs, [percent,falloff], Flags, St),
-    wings_drag:do_drag(Drag, none).
+    wings_drag:setup(Tvs, [percent,falloff], Flags, St).
 
 mag_vertex_tighten(Vs0, We, Magnet, A) ->
     Vis = gb_sets:from_ordset(wings_we:visible(We)),
@@ -1293,28 +1435,23 @@ drag_filter({image,_,_}) ->
     {yes,?__(1,"Drop: Change the texture image")};
 drag_filter(_) -> no.
 
-handle_drop({image,_,#e3d_image{width=W,height=H}=Im}, #st{bb=Uvs0}=St) ->
-    case W =:= H andalso is_power_of_two(W) of
-	false -> keep;
-	true ->
-	    #uvstate{st=GeomSt0,matname=MatName0} = Uvs0,
-	    case MatName0 of 
-		none -> 
-		    wings_u:error(?__(1,"Your are editing the uv-coords of several\n"
-				  "materials, quit the uv-window set all faces\n"
-				  "to the same material"));
-		_ ->
-		    {GeomSt,MatName} = add_material(Im, undefined, MatName0, GeomSt0),
-		    wings_wm:send(geom, {new_state,GeomSt}),
-		    Uvs = Uvs0#uvstate{st=GeomSt,matname=MatName},
-		    get_event(St#st{bb=Uvs})
-	    end
+handle_drop({image,Id,_Im}, #st{bb=Uvs0}=St) ->
+    #uvstate{st=GeomSt0,matname=MatName} = Uvs0,
+    case MatName of 
+	none -> 
+	    Uvs = Uvs0#uvstate{bg_img=Id},
+	    get_event(St#st{bb=Uvs});
+	_ ->
+	    GeomSt = change_texture_id(Id,MatName,GeomSt0),
+	    wings_wm:send(geom, {new_state,GeomSt}),
+	    Uvs = Uvs0#uvstate{st=GeomSt,matname=MatName},
+	    get_event(St#st{bb=Uvs})
     end;
 handle_drop(_DropData, _) ->
     keep.
 
-is_power_of_two(X) ->
-    (X band -X ) == X.
+%% is_power_of_two(X) ->
+%%     (X band -X ) == X.
 
 %%%
 %%% Update charts from new state of Geometry window.
@@ -1357,8 +1494,7 @@ rebuild_charts(We, St = #st{bb=UVS=#uvstate{st=Old,mode=Mode}}, ExtraCuts) ->
 	end,
     Charts2 = auv_segment:cut_model(Charts1, Cuts, We),
     Charts = update_uv_tab(Charts2, FvUvMap),
-    wings_wm:set_prop(wireframed_objects,
-		      gb_sets:from_ordset(lists:seq(1, lists:max([gb_trees:size(Charts),1])))),
+    wings_wm:set_prop(wireframed_objects,gb_sets:from_list(gb_trees:keys(Charts))),
     St#st{sel=[],bb=UVS#uvstate{mode=update_mode(Faces,We),st=Old#st{sel=[]}},
 	  shapes=Charts}.
 
@@ -1677,7 +1813,7 @@ reunfold(Method,#st{sel=Sel,selmode=vertex}=St0) ->
 			      {S,T,_} = gb_trees:get(V, Vtab),
 			      {V,{S,T}}
 			  end || V <- gb_sets:to_list(Vs)],
-		{remap(Method, Pinned, We, St0),I+1}
+		{remap(Method, Pinned, Vs, We, St0),I+1}
 	end,
     {St,_} = wings_sel:mapfold(R, 1, St0),
     wings_pb:done(update_selected_uvcoords(St)).
@@ -1686,42 +1822,39 @@ remap(Method, #st{sel=Sel}=St0) ->
     wings_pb:start(?__(1,"remapping")),
     wings_pb:update(0.001),
     N = length(Sel),
-    Remap = fun(_, We, I) ->
+    Remap = fun(Elems, We, I) ->
 		    Msg = ?__(2,"chart")++" " ++ integer_to_list(I+1),
 		    wings_pb:update(I/N, Msg),
-		    {remap(Method, none, We, St0),I+1}
+		    {remap(Method, none, Elems, We, St0),I+1}
 	    end,
     {St,_} = wings_sel:mapfold(Remap, 1, St0),
     wings_pb:done(update_selected_uvcoords(St)).
 
-remap(stretch_opt, _, We, St) ->
+remap(stretch_opt, _, _, We, St) ->
     Vs3d = orig_pos(We, St),
     ?SLOW(auv_mapping:stretch_opt(We, Vs3d));
-remap(Type, Pinned, #we{name=Ch}=We0, St) ->
-    [Lower,Upper] = wings_vertex:bounding_box(We0),
-    {W,H,_} = e3d_vec:sub(Upper, Lower),
-
+remap(proj_lsqcm, _, Sel, We0, St = #st{selmode=face}) ->
+    Vs3d = orig_pos(We0, St),
+    try Vs0 = auv_mapping:projectFromChartNormal(gb_sets:to_list(Sel),We0#we{vp=Vs3d}),
+	Vtab = gb_trees:from_orddict(Vs0),
+	%%    We1 = #we{vs=Vtab} = update_and_scale_chart(Vs0,We0),
+	SelVs = wings_vertex:from_faces(Sel,We0),
+	Pinned = [begin
+		      {S,T,_} = gb_trees:get(V, Vtab),
+		      {V,{S,T}}
+		  end || V <- SelVs],
+	remap(lsqcm, Pinned, Sel, We0, St)
+    catch throw:What ->
+	    wpa:error(What)
+    end;
+remap(Type, Pinned, _, We0, St) ->
     %% Get 3d positions (even for mapped vs).
     Vs3d = orig_pos(We0, St),
     case auv_mapping:map_chart(Type, We0#we{vp=Vs3d}, Pinned) of
 	{error,Msg} -> 
 	    wpa:error(Msg);
 	Vs0 -> 
-	    We1 = We0#we{vp=gb_trees:from_orddict(sort(Vs0))},
-	    Fs = wings_we:visible(We1),
-	    {{Dx,Dy},Vs1} = auv_placement:center_rotate(Fs, We1),
-	    Center = wings_vertex:center(We0),
-	    Scale = if Dx > Dy -> W / Dx;
-		       true -> H / Dy
-		    end,
-	    Smat = e3d_mat:scale(Scale),
-	    Cmat = e3d_mat:translate(Center),
-	    Fix  = e3d_mat:mul(Cmat, Smat),
-	    Vs = foldl(fun({VId, Pos}, Acc) -> 
-			       [{VId,e3d_mat:mul_point(Fix, Pos)}|Acc] 
-		       end, [], Vs1),
-	    We0#we{vp=gb_trees:from_orddict(sort(Vs)),
-		   name=Ch#ch{size={Dx*Scale, Dy*Scale}}}
+	    update_and_scale_chart(Vs0,We0)
     end.
 
 %% gb_tree of every vertex orig 3d position, (including the new cut ones)
@@ -1738,37 +1871,65 @@ orig_pos(We = #we{name=#ch{vmap=Vmap}},St) ->
 	       end, gb_trees:to_list(We#we.vp)),
     gb_trees:from_orddict(Vs3d).
 
+update_and_scale_chart(Vs0,We0) ->
+    We1 = We0#we{vp=gb_trees:from_orddict(sort(Vs0))},
+    Fs = wings_we:visible(We1),
+    OldA = auv_mapping:fs_area(Fs,We0),
+    NewA = auv_mapping:fs_area(Fs,We1),
+    NewCenter = wings_vertex:center(We1),
+    OldCenter = wings_vertex:center(We0),
+    Scale = math:sqrt(OldA/NewA),
+    T0 = e3d_mat:translate(e3d_vec:neg(NewCenter)),
+    Smat = e3d_mat:scale(Scale),
+    T1  = e3d_mat:mul(Smat, T0),
+    T  = e3d_mat:mul(e3d_mat:translate(OldCenter),T1),
+    wings_we:transform_vs(T, We1).
 
 %%%
 %%% Draw routines.
 %%%
 
-draw_background(#st{mat=Mats,bb=#uvstate{matname=MatN}}) ->
+draw_background(#st{bb=#uvstate{matname=MatName,st=St,bg_img=Image}}) ->
     gl:pushAttrib(?GL_ALL_ATTRIB_BITS),
     wings_view:load_matrices(false),
 
-    %% Draw the background texture.
-
+    %% Draw border around the UV space.
     gl:enable(?GL_DEPTH_TEST),
+    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
+    gl:lineWidth(1),
+    gl:color3f(0, 0, 0.7),
+    gl:translatef(0, 0, -0.5),
+    gl:'begin'(?GL_LINES),
+    G = fun(V) ->
+		gl:vertex2f(V,20),  gl:vertex2f(V,-20),
+		gl:vertex2f(20,V),  gl:vertex2f(-20,V)
+	end,
+    lists:foreach(G, lists:seq(-20,20)),
+    gl:'end'(),
+    gl:lineWidth(3),
+    gl:color3f(0, 0, 1.0),
+    gl:recti(0, 0, 1, 1),
+
+    %% Draw the background texture.
     gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_FILL),
     gl:color3f(1, 1, 1),			%Clear
-    case get({?MODULE,show_background}) andalso has_texture(MatN, Mats) of
+    case get({?MODULE,show_background}) of
 	false -> ok;
-	_ -> wings_material:apply_material(MatN, Mats)
+	_ -> 
+	    Tx = case get_texture(MatName,St) of
+		     false -> wings_image:txid(Image);
+		     DiffId -> wings_image:txid(DiffId)
+		 end,
+	    gl:enable(?GL_TEXTURE_2D),
+	    gl:bindTexture(?GL_TEXTURE_2D, Tx)
     end,
     gl:'begin'(?GL_QUADS),
-    gl:texCoord2f(0, 0),    gl:vertex3f(0, 0, -1),
-    gl:texCoord2f(1, 0),    gl:vertex3f(1, 0, -1),
-    gl:texCoord2f(1, 1),    gl:vertex3f(1, 1, -1),
-    gl:texCoord2f(0, 1),    gl:vertex3f(0, 1, -1),
+    gl:texCoord2f(0, 0),    gl:vertex3f(0, 0, -0.99999),
+    gl:texCoord2f(1, 0),    gl:vertex3f(1, 0, -0.99999),
+    gl:texCoord2f(1, 1),    gl:vertex3f(1, 1, -0.99999),
+    gl:texCoord2f(0, 1),    gl:vertex3f(0, 1, -0.99999),
     gl:'end'(), 
-
-    %% Draw border around the UV space.
-    gl:polygonMode(?GL_FRONT_AND_BACK, ?GL_LINE),
-    gl:lineWidth(2),
-    gl:color3f(0, 0, 1),
-    gl:translatef(0, 0, -0.8),
-    gl:recti(0, 0, 1, 1),
+    gl:disable(?GL_TEXTURE_2D),
 
     gl:popAttrib().
 
