@@ -8,7 +8,7 @@
 %%  See the file "license.terms" for information on usage and redistribution
 %%  of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 %%
-%%     $Id: wings_io.erl,v 1.141 2006/07/13 17:33:07 bjorng Exp $
+%%     $Id: wings_io.erl,v 1.143 2006/08/23 02:56:49 antoneos Exp $
 %%
 
 -module(wings_io).
@@ -79,7 +79,11 @@ set_cursor_1([_|Cs], Name) ->
 
 read_icons() ->
     Ebin = filename:dirname(code:which(?MODULE)),
-    IconFile = filename:join(Ebin, "wings_icon.bundle"),
+    case wings_pref:get_value(interface_icons) of
+	classic -> IconFile = filename:join(Ebin, "wings_icon_classic.bundle");
+	bluecube -> IconFile = filename:join(Ebin, "wings_icon_bluecube.bundle");
+	purpletube -> IconFile = filename:join(Ebin, "wings_icon_purpletube.bundle")
+    end,
     {ok,Bin} = file:read_file(IconFile),
     Bin.
 
